package schedframe.resources.computing.profiles.energy.power.ui;


import java.util.Map;

import schedframe.resources.ResourceStatus;
import schedframe.resources.computing.ComputingResource;
import schedframe.resources.computing.profiles.energy.EnergyEvent;
import schedframe.resources.computing.profiles.energy.EnergyEventType;
import schedframe.resources.computing.profiles.energy.power.PState;
import schedframe.resources.computing.profiles.energy.power.PowerProfile;
import schedframe.resources.computing.profiles.energy.power.PowerStateName;
import schedframe.resources.computing.profiles.energy.power.StandardPowerStateName;

public class ProcessorPowerInterface extends ComputingResourcePowerInterface {

	protected PState currentPState;


	public ProcessorPowerInterface(ComputingResource resource, PowerProfile pp){
		super(resource, pp);
		currentPowerState = StandardPowerStateName.ON;
		currentPState = getSupportedPStates().get("P0");
	}
	
	public boolean setPowerState(PowerStateName powerState){
		if(!supportPowerState(powerState))
			return false;
		if(powerState != currentPowerState){
			currentPowerState = powerState;
			if(powerState == StandardPowerStateName.OFF){
				resource.setStatus(ResourceStatus.UNAVAILABLE);
			} else if(powerState == StandardPowerStateName.ON){
				resource.setStatus(ResourceStatus.FREE);
			}
			resource.handleEvent(new EnergyEvent(EnergyEventType.POWER_STATE_CHANGED, resource.getName()));		
		}

		return true;
	}

	public PState getPState(){
		return currentPState;
	}
	
	public Map<String, PState> getSupportedPStates() {
		return powerProfile.getSupportedPStates();
	}
	
	public boolean supportPState(PState pState) {

		for(String pStateName: getSupportedPStates().keySet()){
			if(pState.getName().equals(pStateName)){
				return true;
			}
		}
		return false;
	}
	
	public boolean setPState(String pStateName){
		PState newPState = getSupportedPStates().get(pStateName);


		if(newPState != null && newPState != currentPState){
			//double factor = newPState.getFrequency()/currentPState.getFrequency();
			currentPState = newPState;
			//CpuSpeed speed = (CpuSpeed)resource.getResourceCharacteristic().getResourceUnits().get(StandardResourceUnitName.CPUSPEED).get(0);
			//speed.setAmount(Double.valueOf(currentPState.getFrequency()).intValue());
			//new ResourceEventCommand(resource).execute(EnergyEventType.FREQUENCY_CHANGED);
			resource.handleEvent(new EnergyEvent(EnergyEventType.FREQUENCY_CHANGED, resource.getName()));
			//resource.getScheduler().sendInternal(GridSimTags.SCHEDULE_NOW, DCWormsTags.UPDATE, resource.getName());
			return true;
		}

		return false;
	}
	
	public double getFrequency() {
		return currentPState.getFrequency();
	}

	public boolean setFrequency(double freq) {

		for(String pStateName: getSupportedPStates().keySet()){
			if(getSupportedPStates().get(pStateName).getFrequency() == freq){
				setPState(pStateName);
				return true;
			}
		}
		return false;
	}

	public double getPowerConsumption(PState state) throws NoSuchFieldException{
		double powerConsumption = 0;
		if(supportPState(state)){
			for(String pStateName: getSupportedPStates().keySet()){
				if(pStateName.equals(state.getName())){
					powerConsumption = getSupportedPStates().get(pStateName).getPower();
					break;
				}
			}
		}
		return powerConsumption;
	}

}

